/*$Id: d_switch.h,v 14.19 1999/08/18 17:17:11 al Exp $ -*- C++ -*-
 * data structures for voltage (and current) controlled switch
 */
#ifndef D_SWITCH_H
#define D_SWITCH_H
#include "e_model.h"
#include "e_elemnt.h"
/*--------------------------------------------------------------------------*/
class MODEL_SWITCH : public MODEL_CARD {
friend class SWITCH_BASE;
private:
  explicit	MODEL_SWITCH(const MODEL_SWITCH& p) 
				:MODEL_CARD(p) {unreachable();}
public:
  explicit	MODEL_SWITCH();
private: // override virtual
  CARD*		clone()const	{untested(); return new MODEL_SWITCH(*this);}
  void		parse(CS&);
  void		print(OMSTREAM,int)const;
private:
  double    vt;		/* threshold voltage */
  double    vh;		/* hysteresis voltage */
  double    ron;	/* on resistance */
  double    roff;	/* off resistance */
  enum control_t {VOLTAGE, CURRENT};
  control_t type;	/* current or voltage controlled */
};
/*--------------------------------------------------------------------------*/
class SWITCH_COMMON : public COMPONENT_COMMON {
public:
  explicit SWITCH_COMMON(int c=0)		:COMPONENT_COMMON(c) {}
  explicit SWITCH_COMMON(const SWITCH_COMMON& p):COMPONENT_COMMON(p) {}
  const char* name()const			{untested(); return "switch";}
};
/*--------------------------------------------------------------------------*/
class SWITCH_BASE : public ELEMENT {
protected:
  explicit	SWITCH_BASE();
  explicit	SWITCH_BASE(const SWITCH_BASE& p);
  void		parse_sb(CS&,int);
  void		expand_sb();
private: // override virtual
  void		print(OMSTREAM,int)const;
  bool		do_tr();
  void		tr_load()	       {tr_load_passive();}
  void		tr_unload()	       {untested(); tr_unload_passive();}
  void		do_ac();
  double	tr_involts()const      {untested(); return tr_outvolts();}
  COMPLEX	ac_involts()const      {untested(); return ac_outvolts();}
  int		numnodes()const	       {return 4;}
  bool		is_1port()const	       {untested(); return true;}
protected:
  std::string	_input_label;		/*this is here instead of in Cswitch*/
  ELEMENT*	_input;		        /* due to bad design and lazyness */
private:
  enum state_t {_UNKNOWN, _ON, _OFF};
  state_t	_ic;		/* initial state, belongs in common */
  state_t	_current_state;	/* state 1 iter ago (may be bad) */
  state_t	_previous_state;/* state 1 time or step ago (known good) */
};
/*--------------------------------------------------------------------------*/
class DEV_VSWITCH : public SWITCH_BASE {
private:
  explicit  DEV_VSWITCH(const DEV_VSWITCH& p):SWITCH_BASE(p){untested();}
public:
  explicit  DEV_VSWITCH()	{}
private: // override virtual
  CARD*	    clone()const	{untested();return new DEV_VSWITCH(*this);}
  void	    parse(CS& cmd)	{parse_sb(cmd,4);}
  void	    expand()		{expand_sb();}
  const char* dev_type()const	{return "vswitch";}
};
/*--------------------------------------------------------------------------*/
class DEV_CSWITCH : public SWITCH_BASE {
private:
  explicit  DEV_CSWITCH(const DEV_CSWITCH& p):SWITCH_BASE(p){untested();}
public:
  explicit  DEV_CSWITCH()	{}
private: // override virtual
  CARD*	    clone()const	{untested();return new DEV_CSWITCH(*this);}
  void	    parse(CS& cmd)	{parse_sb(cmd,2);}
  void	    expand();
  const char* dev_type()const	{return "iswitch";}
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
